#ifndef LEICA_CAMERA_H_
#define LEICA_CAMERA_H_

#include "camera.h"

class LeicaCamera : public Camera
{
public:
  LeicaCamera();
  LeicaCamera(const LeicaCamera &cam);
  ~LeicaCamera() {}

  void set_name(const char* n);
  const char* get_name() const;

  vcl_istream& read_parameters(vcl_istream& in);
  bool get_xyz(vnl_double_2 pUV, vnl_double_3& pXYZ) const;
  bool project(vnl_double_3 const& pXYZ, vnl_double_2& pUV, double* dist2) const;
  vnl_double_3 get_origin() const;
  void setDistCorrectionParams(const vnl_vector_fixed<double,4> &params, double scale);
  double getDist2FromCenter(const vnl_double_2 &imgpt) const;
  Camera *clone() const;

  static void calcDistortionCorrection(LeicaCamera *cam, vnl_vector_fixed<double, 4> &params, double &scale);

  int getWidth() const { return width; }
  int getHeight() const { return height; }
  vnl_double_2 getImageCenter() const { return vnl_double_2(u0, v0); }
  
  double getPixelSizeRatio() const { return Su; }
  double getMetricToPixelU() const { return Du; }
  double getMetricToPixelV() const { return Dv; }

  vnl_double_3 get_axis() const {return R.get_column(2);}

private:

  void read(vcl_istream& in, const char* name, double* value,
            vcl_string* unit = 0);

  int rows, columns;   
  double f, Su, k1, k2, p1, p2;
  double rk1, rk2, rp1, rp2, rscale;
  double Du, Dv;
  double u0, v0;
  int width, height;
};

#endif
