#include "LeicaScan.h"

#include <vul/vul_file_iterator.h>
#include <vul/vul_file.h>
#include <vcl_string.h>
#include <vcl_fstream.h>
#include <vil/vil_load.h>
#include <vil/vil_convert.h>

#include "LeicaRangeData.h"
#include "LeicaCamera.h"

#include "parameters.h"

bool LeicaScan::ReadInScanInfo(const char *directory, const char *scan_name)
{
  // Construct globbing expression.
  if(!directory)
    directory = ".";

  vcl_string dir(directory), expr;

  expr = dir + "/" + scan_name;

  vcl_cout << "Loading " << expr << "\n";

  //remove .ptx from scan_name 
  fullname = vul_file::strip_extension(expr);
  vul_file_iterator filei = expr;
  
  if (filei)  {
    LeicaRangeData *lrd = new LeicaRangeData;
    if (Parameters::Instance()->GetVariable(Parameters::LOADRANGEDATA))
    {
      if (!lrd->load(dir.c_str()))
      {
        vcl_cerr << "Could not load " << dir << "\\rangedata.rd\n";
        lrd->processData(filei());    
        lrd->save(dir.c_str());
      }
    }
    else
    {
      lrd->processData(filei());    
      lrd->save(dir.c_str());
    }

    rangeData = static_cast<RangeData *>(lrd);
  }
  else
  {
    vcl_cout << "Could not find " << filei << vcl_endl;
    return false;
  }
  
  vcl_string imgdir = dir;

  expr = imgdir + "/*.cal";

  int c = 0;
  bool first = true;

  vnl_vector_fixed<double, 4> params;
  double scale;

  int camnum = 0;
  vcl_string outname = imgdir + "/scan_images.txt";
  vcl_ofstream imgnames(outname.c_str());
  // Look for camera calibration files.
  vcl_cout << "Loading Images: ";
  for(vul_file_iterator fi = expr; fi ; ++fi, c++)
  {
    LeicaCamera *lcamera = new LeicaCamera;
    //Read camera parameters.
    vcl_ifstream fin(fi());
    if(fin && lcamera->read_parameters(fin))
    {
      if (first)
      {
        LeicaCamera::calcDistortionCorrection(lcamera, params, scale);
        first = false;
      }

      lcamera->setDistCorrectionParams(params,scale); 

      //Delete images that are not in view
      if (!static_cast<LeicaRangeData *>(rangeData)->isInView(lcamera))
      {
        delete lcamera;
        continue;
      }
      vcl_cout << c << " ";

      vcl_string name = vul_file::strip_extension(vul_file::basename(fi()));
      lcamera->set_name(name.c_str());
      cameras.push_back((Camera *)lcamera);
      vcl_string imgname = imgdir + "/" + name + ".jpg";
      imgnames << camnum << "\t" << imgname << vcl_endl;
      imagenames.push_back(imgname);
      camnum++;
      vil_image_view<vxl_byte> original_image = vil_load(imgname.c_str());
      vil_image_view<double> grey;
      vil_convert_planes_to_grey(original_image, grey);
      images.push_back(grey);   
    }
  }

  imgnames.close();

  vcl_cout << vcl_endl;

  return true;
}
