#ifndef LOCATIONSANDNORMALS_H_
#define LOCATIONSANDNORMALS_H_

#include <vcl_vector.h>
#include <vbl/vbl_array_2d.h>
#include <vnl/vnl_int_2.h>
#include <vnl/vnl_double_2.h>
#include <vnl/vnl_double_3.h>
#include <vnl/vnl_double_2x2.h>
#include <vnl/vnl_double_3x3.h>
#include <vnl/vnl_vector.h>
#include <rrel/rrel_m_est_obj.h>

#include "PreParameters.h"
#include "PointRecord.h"


class LocationsAndNormals {
 public:
  LocationsAndNormals( PreParameters * parameters );
  ~LocationsAndNormals();

  double estimate( vbl_array_2d<PointRecord> & point_grid);

  typedef void (*progress_callback_type)(void*, float);
  void set_progress_callback(void* callback_data,
                             progress_callback_type callback);
  
 private:
  void initialize_grid( vbl_array_2d<PointRecord> & point_grid );
  void generate_initial_grid_locs( vbl_array_2d<PointRecord> & point_grid, 
                                   vcl_vector<vnl_int_2> & initial_grid_locs );
  void initial_estimate_at_point( vbl_array_2d<PointRecord> & point_grid, 
                                  vnl_int_2  grid_loc, 
                                  vcl_vector<vnl_int_2> & pqueue );
  void generate_tangent_and_normal_from_neighbors( vbl_array_2d<PointRecord> const& point_grid,
                                                   vnl_int_2                        grid_loc,
                                                   vnl_double_3x3            const& centering_rotation, 
                                                   vnl_double_3                   & best_tangent,
                                                   vnl_double_3                   & best_other_tangent,
                                                   vnl_double_3                   & normal,
                                                   bool                           & tangent_generated,
                                                   bool                           & normal_generated );
  double estimate_max_scale( vbl_array_2d<PointRecord> & point_grid );
  void generate_final_estimate_at_point( vbl_array_2d<PointRecord> & point_grid, 
                                         vnl_int_2 const& center_grid_loc,
                                         vcl_vector<vnl_int_2> & pqueue );
  bool irls( vbl_array_2d< PointRecord > const& point_grid,
             vcl_vector< vnl_double_3 > const& centered_points,
             vcl_vector< vnl_int_2 >    const& grid_locations,
             vnl_int_2                          center_grid_loc,
             vcl_vector< double >       const& xy_distance_weights,
             vnl_double_3x3              const& uncenter_rotation,
             int                                max_iterations,
             bool                               use_normal_weighting,
             vnl_double_3                     & params,
             double                           & scale,
             double                           & angle_scale,
             vcl_vector< double >            & residuals,
             vcl_vector< double >            & weights,
             vnl_double_2                     & center_2d,
             vnl_double_2x2                   & scatter_2d );
  void reconsider_point_as_surface_or_curve( vbl_array_2d<PointRecord> & point_grid, 
                                             vnl_int_2 grid_loc,
                                             bool last_try );

  void compute_geodesic_scale( vbl_array_2d<PointRecord> & point_grid );
  void form_surface_patches( vbl_array_2d<PointRecord> & point_grid );
  void gather_and_center_points( vbl_array_2d<PointRecord> & point_grid,
                                 vnl_int_2 const& initial_grid_loc,
                                 bool ignore_completed_points, 
                                 bool use_completed_only_when_consistent,
                                 int max_width_in_grid,
                                 vcl_vector< vnl_double_3 > & centered_points,
                                 vcl_vector< vnl_int_2 > & grid_locations,
                                 vnl_double_3x3 & uncenter_rotation );

  void check_pqueue( vbl_array_2d<PointRecord> const& point_grid,
                     vcl_vector< vnl_int_2 > const& pqueue );
  void add_to_pqueue( vbl_array_2d<PointRecord> & point_grid,
                      vcl_vector< vnl_int_2 > & pqueue, vnl_int_2 & grid_loc );
  void remove_from_pqueue( vbl_array_2d<PointRecord> & point_grid,
                           vcl_vector< vnl_int_2 > & pqueue );
  void percolate_up( vbl_array_2d<PointRecord> & point_grid,
                     vcl_vector< vnl_int_2 > & pqueue, vnl_int_2 & grid_loc );

 private:
  PreParameters * m_parameters;
  rrel_m_est_obj * m_est;
  double m_multiplier_for_inlier;
  double m_max_scale_from_initial_estimates;
  double m_max_scale_for_surface_inlier;
  double m_max_allowed_movement_overall;
  double m_max_allowed_movement_scale_multiplier;
  int m_min_inliers_for_surface;
  int m_min_inliers_for_curve;
  void* progress_callback_data;
  progress_callback_type progress_callback;
};



#endif
