#ifndef GRAPH_H_
#define GRAPH_H_

#include "vcl_utility.h"
#include "SimpleRigid.h"
#include <vtkModifiedBSPTree.h>
#include <vcl_set.h>
#include <vcl_string.h>
#include <rsdl/rsdl_point.h>
#include <rsdl/rsdl_kd_tree.h>

#include "Scan.h"

#include "PSKnode.h"
#include "PSKpt.h"

class PSKGraph
{
public:
  
  PSKGraph() : graph_file(NULL) {}
  PSKGraph(Scan &scan, int image_num, const vcl_vector<double> &scales, int left, int right, int top, int bottom, SimpleRigid *xform, bool load_ = false);
  ~PSKGraph();

  bool DecimateToScale(double newscale);
  void DetectExtrema(vcl_vector<PSKpt *> &keypoints);

  void ApplyRigidXForm(const SimpleRigid &sr);
  void WriteVTP(const char *filename) const;
  void GetPointsInRange(const vnl_double_3 &point, double dist, vcl_vector<PSKNode *> &locs) const;
  void GetControlPointsInRange(const vnl_double_3 &point, double dist, vcl_vector<PSKNode *> &locs) const;

  bool ComputeDescriptor(PSKpt *kp);
  
  void SavePSKGraph() const;
  void LoadPSKGraph();
  vcl_string MakeName(double scale);
  double ComputeMinScale(const vcl_vector<double> &scales);
  void ComputeGradients(bool project = false);

  const vcl_vector<PSKNode *> &GetNodes() const { return nodes; }
  int GetMinScaleIndex() const { return minscaleindex; }

private:

  void LoadNextScale();
  void SaveScale() const;
  FILE *graph_file;

  void Delaunay(double scale);
  void Smooth(double scale);
  void DownSampleControl(double thresh);

  PSKpt *RefinePeak(PSKNode *n);
  void ComputeLaplaceBeltrami();
  void IsManifold();
  void ComputeLaplacianBounds(double &lower, double &upper);

  void AssignOrientation(PSKpt *kp);

  vcl_vector<PSKNode *> nodes;
  vcl_vector<int> control;
  vcl_vector<PSKTriangle> triangles;
  double scale;
  vcl_string name;
  rsdl_kd_tree *kdtree;
  rsdl_kd_tree *controlkd;
  bool load;
  double minscale;
  int minscaleindex;

  SimpleRigid *xform;
};

void FilterPSKeypoints(vcl_vector<PSKpt *>::const_iterator begin_itr, vcl_vector<PSKpt *>::const_iterator end_itr,
                       double min_distance_between_points, vcl_vector<PSKpt *> &keypoints3d);

#endif
