#include "PSKnode.h"

//*****************************************************************************

void ConnectTriangle(PSKTriangle &t, int tindex)
{
  double ablen = (t.a->point - t.b->point).two_norm();
  double aclen = (t.a->point - t.c->point).two_norm();
  double bclen = (t.b->point - t.c->point).two_norm();

  double s = (ablen + aclen + bclen)/2.0;
  t.area = sqrt(s*(s-ablen)*(s-aclen)*(s-bclen));

  t.a->triangles.push_back(tindex);
  t.b->triangles.push_back(tindex);
  t.c->triangles.push_back(tindex);

  t.a->adjacent.push_back(t.b);
  t.a->adjacent.push_back(t.c);

  t.b->adjacent.push_back(t.a);
  t.b->adjacent.push_back(t.c);

  t.c->adjacent.push_back(t.a);
  t.c->adjacent.push_back(t.b);
}

//*****************************************************************************

bool RayIntersectTriangle(const PSKTriangle &t, const vnl_double_3 &pt, const vnl_double_3 &dir)
{
	bool intersect = true;
	
  vnl_double_3 direction = dir;
  direction.normalize();
	//find point on the plane of the triangle
  vnl_double_3 normal = vnl_cross_3d((t.b->point - t.a->point),(t.c->point - t.a->point)).normalize();
	double d = -dot_product(normal, t.a->point);
	double s = (-d - dot_product(pt, normal)) / (dot_product(direction, normal));

	if (s <= 1e-12)	//intersection is behind the ray
	  return false;

	vnl_double_3 p = pt + s*direction;

	//Now we must check if point is valid
	vnl_double_3 v1 = (t.a->point - p), v2 = (t.b->point - p), v3 = (t.c->point - p);

	if (dot_product(vnl_cross_3d(v1, v2), normal) < 0.0 || dot_product(vnl_cross_3d(v2, v3),normal) < 0.0 ||
		  dot_product(vnl_cross_3d(v3, v1), normal) < 0.0 )
	  return false;
	
  return true;
}

//*****************************************************************************
