#include "PSKpt.h"

#include <vcl_map.h>
#include <vnl/vnl_double_3.h>
#include <vcl_iostream.h>

//*****************************************************************************

void WriteKeypoint(FILE *file, PSKpt *kp)
{
  fwrite(kp->location.data_block(), sizeof(double)*3, 1, file); 
  fwrite(kp->direction.data_block(), sizeof(double)*3, 1, file);
  fwrite(kp->normal.data_block(), sizeof(double)*3, 1, file);
  fwrite(&kp->scale, sizeof(double), 1, file); 
  fwrite(&kp->strength, sizeof(double), 1, file);
  fwrite(&kp->id, sizeof(int), 1, file); 
  fwrite(kp->feature_vector.data_block(), sizeof(double)*128, 1, file);

  double dh = 4.0 * kp->scale;   //bin width
  double radius = dh*2.0;                 //radius of descriptor

  vnl_double_3 binormal = vnl_cross_3d(kp->normal, kp->direction);

  vnl_double_3 ul, ur, ll, lr;
  ul = kp->location + radius*binormal - radius*kp->direction;
  ur = kp->location + radius*binormal + radius*kp->direction;
  ll = kp->location - radius*binormal - radius*kp->direction;
  lr = kp->location - radius*binormal + radius*kp->direction;

  fwrite(ul.data_block(), sizeof(double)*3, 1, file);
  fwrite(ur.data_block(), sizeof(double)*3, 1, file);
  fwrite(ll.data_block(), sizeof(double)*3, 1, file);
  fwrite(lr.data_block(), sizeof(double)*3, 1, file);
}

//*****************************************************************************

void ReadKeypoint(FILE *file, PSKpt *kp)
{
  fread(kp->location.data_block(), sizeof(double)*3, 1, file); 
  fread(kp->direction.data_block(), sizeof(double)*3, 1, file);
  fread(kp->normal.data_block(), sizeof(double)*3, 1, file);
  fread(&kp->scale, sizeof(double), 1, file);
  fread(&kp->strength, sizeof(double), 1, file);
  fread(&kp->id, sizeof(int), 1, file);
  fread(kp->feature_vector.data_block(), sizeof(double)*128, 1, file);

  vnl_double_3 ul, ur, ll, lr;
  fread(ul.data_block(), sizeof(double)*3, 1, file);
  fread(ur.data_block(), sizeof(double)*3, 1, file);
  fread(ll.data_block(), sizeof(double)*3, 1, file);
  fread(lr.data_block(), sizeof(double)*3, 1, file);
}

//*****************************************************************************

void WriteKeypoints(const KeyVec &kpts, const char *filename)
{
  FILE *file = fopen(filename, "wb");
  unsigned int numscales = (unsigned int)kpts.size();
  fwrite(&numscales, sizeof(unsigned int), 1, file);

  int count = 0;
  for (unsigned int i = 0; i < numscales; i++)
  {
    unsigned int size = (unsigned int)kpts[i].size();
    fwrite(&size, sizeof(unsigned int), 1, file);
    int kpid = 0;
    for (unsigned int j = 0; j < size; j++)
    {
      PSKpt *key = kpts[i][j];
      key->id = kpid++;
      count++;
      WriteKeypoint(file, key);
    }
  }

  fclose(file);
  vcl_cout << "Wrote " << count << " keypoints to " << filename << vcl_endl;
}

//*****************************************************************************

void ReadKeypoints(KeyVec &kpts, const char *filename)
{
  kpts.clear();
  FILE *file = fopen(filename, "rb");
  unsigned int numscales;
  fread(&numscales, sizeof(unsigned int), 1, file);

  kpts.resize(numscales);
  int count = 0;
  for (unsigned int i = 0; i < numscales; i++)
  {
    unsigned int size;
    fread(&size, sizeof(unsigned int), 1, file);
    kpts[i].resize(size);
    for (unsigned int j = 0; j < size; j++)
    {
      kpts[i][j] = new PSKpt;
      ReadKeypoint(file, kpts[i][j]);
      count++;
    }
  }

  fclose(file);
  vcl_cout << "Read " << count << " keypoints from " << filename << vcl_endl;
}

//*****************************************************************************

void WriteKeypoints(const vcl_vector<PSKpt *> &kpts, const char *filename)
{
  FILE *file = fopen(filename, "wb");

  unsigned int size = (unsigned int)kpts.size();
  fwrite(&size, sizeof(unsigned int), 1, file);
  int kpid = 0;
  for (unsigned int j = 0; j < size; j++)
  {
    PSKpt *key = kpts[j];
    key->id = kpid++;
    WriteKeypoint(file, key);
  } 

  fclose(file);
  vcl_cout << "Wrote " << kpid << " keypoints to " << filename << vcl_endl;
}

//*****************************************************************************

void ReadKeypoints(vcl_vector<PSKpt *> &kpts, const char *filename)
{
  kpts.clear();
  FILE *file = fopen(filename, "rb");

  unsigned int size;
  fread(&size, sizeof(unsigned int), 1, file);
  kpts.resize(size);
  for (unsigned int j = 0; j < size; j++)
  {
    kpts[j] = new PSKpt;
    ReadKeypoint(file, kpts[j]);
  }

  fclose(file);
  vcl_cout << "Read " << size << " keypoints from " << filename << vcl_endl;
}

//*****************************************************************************

void FreeKeyVec(KeyVec &kv)
{
  for (unsigned int i = 0; i < kv.size(); i++)
    for  (unsigned int j = 0; j < kv[i].size(); j++)
      delete kv[i][j];
}

//*****************************************************************************

void FreeKeyVec(vcl_vector<PSKpt *> &kv)
{
  for  (unsigned int j = 0; j < kv.size(); j++)
    delete kv[j];
}

//*****************************************************************************
