
#include <vnl/vnl_math.h>
#include "PointRecord.h"

void print( std::ostream & str, PointRecord const& p )
{
  str << "is_valid_return = " << p.is_valid_return
      << ", direction = " << p.direction;
  if ( !p.is_valid_return )
    {
      str << std::endl;
      return;
    }

  str << ", measured_distance = " << p.measured_distance  << '\n'
      << "point_spacing_normal_to_los = " << p.point_spacing_normal_to_los
      << ", is_initialized = " << p.is_initialized
      << ", is_done = " << p.is_done;

  str << "\n" << "surface_type = ";
  switch ( p.surface_type ) 
    {
    case PointRecord::PRSurface : str << "PRSurface"; break;
    case PointRecord::PRCurve : str << "PRCurve"; break;
    case PointRecord::PRCloud : str << "PRCloud"; break;
    case PointRecord::PRUndersampled : str << "PRUndersampled"; break;
    case PointRecord::PROutlier : str << "PROutlier"; break;
    };
  if ( p.surface_type == PointRecord::PROutlier )
    {
      str << std::endl;
      return;
    }
  double dot = dot_product( p.surface_normal, -p.direction );
  if ( dot < -1 ) dot = -1;
  if ( dot > 1 ) dot = 1;
  double angle = acos(dot) * 180 / vnl_math::pi;
  str << ", estimated_scale = " << p.estimated_scale 
      << ", estimated_angle_scale (degrees) = " << p.estimated_angle_scale * 180 / vnl_math::pi
      << ", estimated_distance = " << p.estimated_distance << '\n'
      << "surface_normal = " << p.surface_normal 
      << ", angle between normal and los (degrees) = " << angle << std::endl;

  str << "corner directions and distances from center:  \n"
      << "  (0):"
      << (p.corner_directions[0] - p.direction) / (p.corner_directions[0] - p.direction).magnitude()
      << ";  " << p.corner_distances[0] << '\n'
      << "  (1):"
      << (p.corner_directions[1] - p.direction) / (p.corner_directions[1] - p.direction).magnitude()
      << ";  " << p.corner_distances[1] << '\n'
      << "  (2):"
      << (p.corner_directions[2] - p.direction) / (p.corner_directions[2] - p.direction).magnitude()
      << ";  " << p.corner_distances[2] << '\n'
      << "  (3):"
      << (p.corner_directions[3] - p.direction) / (p.corner_directions[3] - p.direction).magnitude()
      << ";  " << p.corner_distances[3] << '\n';
}
