#include <iostream>
#include <vnl/vnl_math.h>
#include "PreParameters.h"

PreParameters::PreParameters()
{
  sensor_sigma = 0.002;   //  sensor noise
  //sensor_sigma = 0.004;   //  sensor noise

  initial_region_max_grid_width = 9; //15
  multiplier_on_std_dev_for_max_scale = 4.5;

  max_irls_iterations = 5;
  max_irls_scale_iterations = 3;
  use_weighting_based_on_normals = true;

  min_ratio_width_to_noise_for_surface = 2.5;
  min_inliers_for_fit = 9;
  min_inliers_for_curve = min_inliers_for_fit - 2;
  max_sample_distance_perpendicular_to_los = 8 * 0.0254;  // 4 inches

  min_angle_delta_for_convergence = 1.0 * vnl_math::pi / 180.0;  // degrees
  intercept_mult_for_convergence = 0.05;  // times the noise

  tukey_parameter = 4.5;
  min_component_size = min_inliers_for_fit;
}

void
PreParameters::print( std::ostream & str )
{
  str << "sensor_sigma = " << sensor_sigma << '\n'
      << "multiplier_on_std_dev_for_max_scale = " << multiplier_on_std_dev_for_max_scale << '\n'
      << "initial_region_max_grid_width = " << initial_region_max_grid_width << '\n'
      << "max_irls_iterations = " << max_irls_iterations << '\n'
      << "max_irls_scale_iterations = " << max_irls_scale_iterations << '\n'
      << "use_weighting_based_on_normals = " << use_weighting_based_on_normals << '\n'
      << "min_ratio_width_to_noise_for_surface = " << min_ratio_width_to_noise_for_surface << '\n'
      << "min_inliers_for_fit = " << min_inliers_for_fit << '\n'
      << "min_inliers_for_curve = " << min_inliers_for_curve << '\n'
      << "max_sample_distance_perpendicular_to_los = " << max_sample_distance_perpendicular_to_los << '\n'
      << "min_angle_delta_for_convergence = " << min_angle_delta_for_convergence << '\n'
      << "intercept_mult_for_convergence = " << intercept_mult_for_convergence << '\n'
      << "tukey_parameter = " << tukey_parameter << '\n'
      << "min_component_size = " << min_component_size << '\n';
}
