
#ifndef PreParameters_h_
#define PreParameters_h_

class PreParameters {
 public:
  PreParameters();
 
  double sensor_sigma;

  //  Max grid region width over which to gather points to estimate the
  //  transformation. 
  int initial_region_max_grid_width;

  double multiplier_on_std_dev_for_max_scale;

  //  Maximum iterations in IRLS surface fitting.
  int max_irls_iterations;

  //  Maximum iterations in IRLS surface fitting during which scale is
  //  to be re-estimated.
  int max_irls_scale_iterations;

  //  True => use differences between normals in IRLS weighting in
  //  addition to using the usual distance.
  bool use_weighting_based_on_normals;

  //  When the spread of the points on the surface is less than this
  //  multiplier times the apparent noise, the point is treated as
  //  though it is part of a cloud (e.g. tree)
  double min_ratio_width_to_noise_for_surface;

  //  Minimum number of inliers for computing a least-squares fit.
  //  When the inlier count goes below this, the point is considered
  //  an outlier (or perhaps part of a point cloud).
  int min_inliers_for_fit; 
  int min_inliers_for_curve; 

  //  When the distance between samples perpendicular to the line of
  //  sight is greater than this distance, the points are treated as
  //  undersampled.  Think of this as the "steps" example.
  double max_sample_distance_perpendicular_to_los;


  //  When the change in the estimated planar surface normal is below
  //  this threshold, the estimation of the normal can be considered
  //  converged.
  double min_angle_delta_for_convergence;  // in radians

  //  When the change in the intercept of the planar surface equation
  //  is below a threshold formed by multiplying this parameter times
  //  the scale, the estimation of the intercept can be considered
  //  converged.
  double intercept_mult_for_convergence;

  //  Parameter for the Tukey biweight function.
  double tukey_parameter;  //  4.5

  //  Minimum number of points in a connected component of points that
  //  are not marked done.  This is used in the second and subsequent
  //  iterations of surface normal estimation.
  int min_component_size;

  void print( std::ostream & str );
};

#endif
