#ifndef RANGEDATA_H_
#define RANGEDATA_H_

#include <vnl/vnl_double_3.h>
#include <vcl_vector.h>

class RangeData
{
public:
  RangeData() {}
  virtual ~RangeData() {}

  ///Read scan data from a file, specified by name
  virtual bool processData(const char *filename) = 0;

  ///back projects a point (image_pt) into the range data provided the origin of the camera that the
  ///image was taken from, and returns the projected point
  virtual bool backProject(const vnl_double_3 &image_pt, const vnl_double_3 &cam_origin,
                       vnl_double_3 &projected_pt, unsigned int *pt_index) const = 0;

  ///Index of point closest to pt
  virtual unsigned int findClosestTo(const vnl_double_3 &pt) const = 0;

  const vcl_vector< vnl_double_3 > &get_points() const { return points; }
  const vcl_vector< vnl_double_3 > &get_normals() const { return normals; }
  vcl_vector< vnl_double_3 > &get_normals()  { return normals; }
  double getStdDev() const { return std_dev; }

protected:

  vcl_vector<vnl_double_3> points;
  vcl_vector<vnl_double_3> normals;
  double std_dev; //noise estimate of range data
};

#endif