#include "Scan.h"
#include "parameters.h"

#include <vnl/vnl_cross.h>
#include <vnl/vnl_math.h>
#include <vnl/algo/vnl_symmetric_eigensystem.h>

#include <vgl/vgl_polygon_scan_iterator.h>

#include <vcl_utility.h>
#include <vcl_map.h>
#include <vcl_algorithm.h>
#include <float.h>

#include <rsdl/rsdl_bins.h>

#include "vtkFloatArray.h"
#include "vtkPoints.h"
#include "vtkPolyData.h"
#include "vtkPointData.h"
#include "vtkSmartPointer.h"
#include "vtkXMLPolyDataWriter.h"
#include "vtkCellArray.h"
#include "vtkCharArray.h"
#include "vtkLine.h"
#include "vtkCellData.h"

#include "PSKgraph.h"

// Hard-coded parameters for scale space construction.  These could be
// converted to run-time parameters.
#define DESCRIPTOR_COMPONENT_MAXIMUM 0.2

#include "utilities.h"

// The factor by which to multiply the scale to get the circle radius.
const double radiusFactor = 2.0;

//****************************************************************************/

Scan::Scan() : rangeData(NULL)
{

}

//****************************************************************************/

Scan::~Scan()
{
  if (rangeData)
    delete rangeData;
  
  for (unsigned int i = 0; i < cameras.size(); i++)   
    delete cameras[i];
  
  for (unsigned int s = 0; s < psks.size(); s++)
    for (unsigned int i = 0; i < psks[s].size(); i++)
      delete psks[s][i];
}

//****************************************************************************/

KeyVec &Scan::ComputePSK(const vcl_vector<double> &scales)
{  
  KeyVec kpts;

  kpts.resize(scales.size());

  size_t count = 0;
  for (unsigned int im = 0; im < images.size(); im++)
  {
    vcl_cout << "\n" << this->GetFullScanName() << "\n";
    vcl_cout << "COMPUTING KPTS FOR IMAGE: " << im+1 << "\n";
    PSKGraph *graph = new PSKGraph(*this, im, scales, -1, -1, -1, -1, NULL, false);
    if (graph->GetNodes().empty()) {
      delete graph;
      continue;
    }
       
    unsigned int i = graph->GetMinScaleIndex();
    graph->DetectExtrema(kpts[i]); // detect for minscale
    count += kpts[i].size(); 
    i++;

    for ( ; i < scales.size(); i++)
    {
      graph->DecimateToScale(scales[i]);
      graph->DetectExtrema(kpts[i]);
      count += kpts[i].size();
    }

    delete graph;
  }

  vcl_cout << "Computed " << count << " keypoints across " << scales.size() << " scales.\n";

  count = 0;
  psks.resize(scales.size());
  for (unsigned int s = 0; s < scales.size(); s++)
  {
    double filterwidth = 3.0*scales[s];
    FilterPSKeypoints(kpts[s].begin(), kpts[s].end(), filterwidth, psks[s]);
    count += psks[s].size();
  }

  vcl_cout << "After filtering computed " << count << " keypoints.\n";

  return psks;
}

//****************************************************************************/

vcl_vector<Camera *> Scan::CopyCameras() const
{
  vcl_vector<Camera *> newcameras;
  for (unsigned int i = 0; i < cameras.size(); i++)
    newcameras.push_back(cameras[i]->clone());
  return newcameras;
}

//****************************************************************************/
