#ifndef SCAN_H_
#define SCAN_H_

#include "RangeData.h"
#include "camera.h"

#include <rsdl/rsdl_kd_tree.h>
#include <vil/vil_image_view.h>

class PSKpt;
typedef vcl_vector<vcl_vector<PSKpt *> > KeyVec;

class Scan
{
public:

  Scan();
  virtual ~Scan();

  virtual bool ReadInScanInfo(const char *directory, const char *scan_name) = 0;
  KeyVec &ComputePSK(const vcl_vector<double> &scales);

  const KeyVec &GetPSKs() const { return psks; }
  KeyVec &GetPSKs() { return psks; }

  RangeData *GetRangeData() const { return rangeData; }
  const vcl_vector<Camera *> &GetCameras() const { return cameras; }  
  vcl_vector<Camera *> CopyCameras() const;
  const vcl_vector<vil_image_view<double> > &GetImages() const { return images; }

  vcl_string GetFullScanName() const { return fullname; }

protected:

  RangeData *rangeData;
  vcl_vector<Camera *> cameras;
  vcl_vector<vil_image_view<double> > images;   //indexes parallel to cameras
  vcl_vector<vcl_string> imagenames;
  KeyVec psks;
  vcl_string fullname;
};

bool operator<(const vnl_double_3 &left, const vnl_double_3 &right);

#endif
