#ifndef SPHERICAL_BINS_H_
#define SPHERICAL_BINS_H_

#include <vcl_vector.h>
#include <vnl/vnl_double_3.h>

class SphericalBins
{
public:

  SphericalBins() :  m_bins_width(0), m_theta_origin(0.0),
                     m_phi_origin(0.0), m_theta_spacing(0.0), 
                     m_phi_spacing(0.0)
  {}

  SphericalBins(int bins_height, int bins_width, double theta_origin, 
                double phi_origin, double theta_spacing, double phi_spacing) 
                : m_bins_height(bins_height), m_bins_width(bins_width), m_theta_origin(theta_origin), 
                m_phi_origin(phi_origin), m_theta_spacing(theta_spacing), 
                m_phi_spacing(phi_spacing) 
  { m_bins.resize(bins_height * bins_width); }

  const vcl_vector<unsigned int>& getBinAtIndex(unsigned int index) const;
  int getBinAtThetaPhi(double theta, double phi, vcl_vector<unsigned int>& bin);
  int getThetaIndex(double theta) const;
  int getPhiIndex(double phi) const;

  bool insertIndex(double theta, double phi, unsigned int ind);
  bool insertIndex(unsigned int index, unsigned int ind);
  unsigned int findClosestTo(const vnl_double_3 &pt, const vcl_vector<vnl_double_3> &points, double thresh) const;

  const vcl_vector< vcl_vector<unsigned int> > &getBins() const { return m_bins; }
  double getThetaSpacing() const { return m_theta_spacing; }
  double getPhiSpacing() const { return m_phi_spacing; }
  double getThetaOrigin() const { return m_theta_origin; }
  double getPhiOrigin() const { return m_phi_origin; }
  int getBinsWidth() const { return m_bins_width; }
  int getBinsHeight() const { return m_bins_height; }

private:

  vcl_vector< vcl_vector<unsigned int> > m_bins;
  int m_bins_height;
  int m_bins_width;
  double m_theta_origin;
  double m_phi_origin;
  double m_theta_spacing;
  double m_phi_spacing;
};


#endif
