#ifndef CAMERA_H_
#define CAMERA_H_

#include <vnl/vnl_matrix_fixed.h>
#include <vnl/vnl_double_3.h>
#include <vnl/vnl_double_2.h>

#include <vcl_iosfwd.h>
#include <vcl_string.h>
#include <vcl_vector.h>

class Camera
{
public:

  Camera() : name("") {}
  Camera(const Camera &cam) {
    name = cam.name;  R = cam.R;
    t = cam.t;        origin = cam.origin;
  }

  virtual void set_name(const char* n) = 0;
  virtual const char* get_name() const = 0;

  virtual vcl_istream& read_parameters(vcl_istream& in) = 0;
  virtual bool get_xyz(vnl_double_2 pUV, vnl_double_3& pXYZ) const = 0;
  virtual bool project(vnl_double_3 const& pXYZ, vnl_double_2& pUV, double* dist2) const = 0;
  virtual vnl_double_3 get_origin() const = 0;
  virtual double getDist2FromCenter(const vnl_double_2 &imgpt) const = 0;
  virtual Camera *clone() const = 0;
  virtual vnl_double_3 get_axis() const = 0;
  vnl_matrix_fixed<double, 3, 3> get_R() const { return R; }
  
protected:
  vcl_string name;
  vnl_matrix_fixed<double, 3, 3> R;
  vnl_double_3 t, origin;     //origin is equal to -R^T * t
};
#endif
