#include "utilities.h"

#include "vtkFloatArray.h"
#include "vtkPoints.h"
#include "vtkPolyData.h"
#include "vtkPointData.h"
#include "vtkSmartPointer.h"
#include "vtkXMLPolyDataWriter.h"

const double twoPi = 2*vnl_math::pi;

//*****************************************************************************

vnl_double_3 Utilities::cartesianToSpherical(const vnl_double_3 &pt)
{
  double x = pt[0], y = pt[1], z = pt[2];
  double r = vcl_sqrt(x*x+y*y+z*z);
  double theta = 0.0;
  double phi = 0.0;

  if(r > 0)  
    phi = vcl_acos(z/r);

  if(x*x+y*y > 0)
    theta = vcl_atan2(y, x);

  if (theta < 0.0)
    theta += twoPi;

  return vnl_double_3(r, theta, phi);
}

//*****************************************************************************

vnl_double_3 Utilities::sphericalToCartesian(const vnl_double_3 &sphericalPt)
{
  vnl_double_3 point;
  point[0] = sphericalPt[0] * vcl_cos(sphericalPt[1]) * vcl_sin(sphericalPt[2]);
  point[1] = sphericalPt[0] * vcl_sin(sphericalPt[1]) * vcl_sin(sphericalPt[2]);
  point[2] = sphericalPt[0] * vcl_cos(sphericalPt[2]);
  return point;
}

//*****************************************************************************

int Utilities::classifyPoint(const vnl_double_3 &normal, double d, const vnl_double_3 &pt)
{
    double p = dot_product(normal, pt) + d;
    if( p > 0.0f )
      return 1;
    else if( p < 0.0f )
      return -1;
    else
      return 0;
}

//*****************************************************************************

int Utilities::classifyPoint(const vnl_double_4 &plane, const vnl_double_3 &pt)
{
    vnl_double_3 normal(plane[0], plane[1], plane[2]);
    double p = dot_product(normal, pt) + plane[3];
    if( p > 0.0f )
      return 1;
    else if( p < 0.0f )
      return -1;
    else
      return 0;
}

//*****************************************************************************
