#include "PSKgraph.h"
#include "Scan.h"
#include "parameters.h"
#include <vcl_algorithm.h>
#include <vcl_map.h>

#include "vtkSmartPointer.h"
#include "vtkPolyData.h"
#include "vtkPointData.h"
#include "vtkXMLPolyDataWriter.h"
#include "vtkCellArray.h"
#include "vtkIntArray.h"
#include "vtkLine.h"

#include <rrel/rrel_cauchy_obj.h>
#include <rsdl/rsdl_point.h>
#include <rsdl/rsdl_kd_tree.h>
#include <vnl/vnl_random.h>
#include <rsdl/rsdl_bins.h>

#include <vul/vul_arg.h>
#include <vul/vul_file.h>

#include "ScanFactory.h"
#include "MatchPSK.h"

#define MINSCALE .03

//Computes physical scale keypoints for two scans then matches them computing transformations from scan2 to scan1
int main(int argc, char *argv[])
{
  vul_arg<vcl_string> path1(0, "Full path to the first scan (*.ptx file)");
  vul_arg<vcl_string> path2(0, "Full path to the second scan (*.ptx file)");
  vul_arg<double> start_scale("-ss", "the smallest scale", MINSCALE);
  vul_arg<int> num_scales("-ns", "number of scales to compute", 6);
  vul_arg<bool> load_kpts("-lk", "load pre-computed kpts", false);
  vul_arg_parse(argc, argv);

  Parameters::Instance()->SetVariable(Parameters::LOADRANGEDATA, true);

  vcl_string dir1 = vul_file::dirname(path1());
  vcl_string name1 = vul_file::strip_directory(path1());
  vcl_string dir2 = vul_file::dirname(path2());
  vcl_string name2 = vul_file::strip_directory(path2());

  KeyVec fixedkpts, movingkpts;

  vcl_vector<double> scales;
  scales.push_back(start_scale());
  double k = pow(2.0, 1.0/2.0);
  for (int i = 1; i < num_scales(); i++)
    scales.push_back(k * scales.back());

  fixedkpts.resize(scales.size());
  movingkpts.resize(scales.size());

  Scan *scan1 = ScanFactory::create_scan(ScanFactory::LEICA);
  scan1->ReadInScanInfo(dir1.c_str(), name1.c_str()); 

  Scan *scan2 = ScanFactory::create_scan(ScanFactory::LEICA);
  scan2->ReadInScanInfo(dir2.c_str(), name2.c_str());

  if (!load_kpts())
  {
    scan1->ComputePSK(scales);
    scan2->ComputePSK(scales);
  }
  else
  {
    ReadKeypoints(scan1->GetPSKs(), (dir1 + "/pskpts.dat").c_str());
    ReadKeypoints(scan2->GetPSKs(), (dir2 + "/pskpts.dat").c_str());
  }

  vcl_vector<SimpleRigid> xforms;
  vcl_vector<pskmatch> matches;
  ComputeXFormsPSK(scan2, scan1, xforms, matches);

  WriteKeypoints(scan1->GetPSKs(), (dir1 + "/pskpts.dat").c_str());
  WriteKeypoints(scan2->GetPSKs(), (dir2 + "/pskpts.dat").c_str());
  WritePSKMatches(matches, scan1->GetPSKs(), (dir2 + "/matches_to_" + vul_file::strip_extension(name1) + ".dat").c_str());

  vcl_ofstream xformfile((dir2 + "/xforms_to_" + vul_file::strip_extension(name1) + ".txt").c_str());
  xformfile << path1() << "\n" << path2() << "\n" << xforms.size() << "\n";
  for (unsigned int i = 0; i < xforms.size(); i++)
  {
    xforms[i].write_to_xform_file(xformfile);
  }

  xformfile.close();

  return 0;
}

//*****************************************************************************
