#define MAX_XFORMS 50

#include "parameters.h"
#include "mesh.h"
#include "PSKgraph.h"

#include <vnl/vnl_random.h>
#include <vul/vul_file.h>

#include "ScanFactory.h"

#include "utilities.h"

int main(int argc, const char *argv[])
{
  vcl_string dir = vcl_string(argv[1]);

  vcl_ifstream infile(argv[2]);
  unsigned int numscans;
  infile >> numscans;

  double noise = 0.002;
  Parameters::Instance()->SetNoise(noise);
  
  for (unsigned int i = 0; i < numscans; i++)
  {
    vcl_string subdir, scanname;
    infile >> subdir >> scanname;

    vcl_string indir = dir + "/" + subdir;

    vcl_cout << indir + "/" + scanname+".ptx" << vcl_endl;

    Scan *scan = ScanFactory::create_scan(ScanFactory::LEICA);
    scan->ReadInScanInfo((indir).c_str(), (scanname+".ptx").c_str());

    vcl_cout << "Number of points: " << scan->GetRangeData()->get_points().size() << "\n";
    vcl_cout << "Number of cameras: " << scan->GetCameras().size() << "\n";
        
    delete scan;

    MeshRangeData *mrd = new MeshRangeData;
    mrd->ReadInScanInfo((indir).c_str(), (scanname+".ptx").c_str());  
    Mesh mesh(mrd);
    mesh.WriteToVTP((indir+"/"+scanname+".vtp").c_str());

    //remove pointgrid file
    vul_file::delete_file_glob((indir+"/"+scanname+".pg").c_str());
  }

  infile.close();
  

  return 0;
}

//*****************************************************************************