//Eric Smith
#include "Input.h"
#include <cmath>
#include <vcl_iostream.h>
#include <vcl_string.h>
#include <vcl_algorithm.h>

#include <vxl_config.h>
#include <vil/vil_rgb.h>
#include <vil/vil_save.h>
#include <vil/vil_image_view.h>
#include <vil/vil_convert.h>

//Utility macros
#define DEGREES_TO_RADIANS(x) (x)*0.017453292
#define RADIANS_TO_DEGREES(x) (x)*57.29577951

bool mousedown = false;
int oldx, oldy;
float speed = .25f;
bool fast = true;

//*****************************************************************************

void KeyboardFunction(GLubyte key, GLint x, GLint y)
{
  if (key == 27)  //quit if esc
    exit(0);
     
  HandleInput(key);
  
  glutPostRedisplay();
}

//*****************************************************************************

void KeyboardUpFunction(GLubyte key, GLint x, GLint y)
{
  //key no longer pressed
  glutPostRedisplay();
}

//*****************************************************************************

void MouseFunction(int button, int state, int x, int y)
{
  //On left mouse down
  if (button == GLUT_RIGHT_BUTTON && state == GLUT_DOWN)
  {
    //mouse is down flag is true and save location
    mousedown = true;
    oldx = x;
    oldy = y;    
  }
  else if (button == GLUT_RIGHT_BUTTON)
    mousedown = false;
}

//*****************************************************************************

void MotionFunction(int x, int y)
{
  if (mousedown)  //rotate only if mouse button is down
  {
    //find how far we moved and in what direction
    int xdist = x - oldx;
    int ydist = y - oldy;

    Data::Inst().eyeangle -= (float)xdist/100.0f;
    if (Data::Inst().eyeangle < 0.0f)
      Data::Inst().eyeangle +=  (float)(DEGREES_TO_RADIANS(360));

    //Store current position
    oldx = x;
    oldy = y;
  }
}

//*****************************************************************************

void HandleInput(GLubyte key)
{
  switch (key)
  {
    case 'w':
    case 'W':
    {
      //Move foward in eyeangle direction
      Data::Inst().eyex += speed * (float)std::cos(Data::Inst().eyeangle);  
      Data::Inst().eyey += speed *  (float)std::sin(Data::Inst().eyeangle);
      break;
    }
    case 's':
    case 'S':
    {
      //move backwards in eyeangle direction
      Data::Inst().eyex -= speed *  (float)std::cos(Data::Inst().eyeangle);
      Data::Inst().eyey -= speed *  (float)std::sin(Data::Inst().eyeangle);
      break;
    }  
    case 'a':
    case 'A':
    {
      //strafe left while looking at eyeangle direction
      Data::Inst().eyex -= speed *  (float)std::sin(Data::Inst().eyeangle);
      Data::Inst().eyey += speed *  (float)std::cos(Data::Inst().eyeangle);
      break;
    }
    case 'd':
    case 'D':
    {
      //strafe right while looking at eyeangle direction
      Data::Inst().eyex += speed * (float)std::sin(Data::Inst().eyeangle);
      Data::Inst().eyey -= speed * (float)std::cos(Data::Inst().eyeangle);
      break;
    }
    case 'q':
    case 'Q':
    {
      Data::Inst().eyez += speed;
      break;
    }
    case 'e':
    case 'E':
    {
      Data::Inst().eyez -= speed;
      break;
    }
    case 'c':
    case 'C':
    {
      Data::Inst().fixedmesh.FlipColor();
      Data::Inst().movingmesh.FlipColor();
      break;
    }
    case 'f':
    {
      Data::Inst().drawfixed = !Data::Inst().drawfixed;
      break;
    }
    case 'g':
    {
      Data::Inst().drawmoving = !Data::Inst().drawmoving;
      break;
    }
    case 'r':
    case 'R':
    {
      Data::Inst().drawfixed = true;
      Data::Inst().drawmoving = true;
      Data::Inst().fixedmesh.LookAt();
      break;
    }
    case ',':
    case '<':
    {
      Data::Inst().xform_i--;
      if (Data::Inst().xform_i  < 0)
        Data::Inst().xform_i = (int)Data::Inst().GetXForms().size() - 1;
      vcl_cout << "Xform: " << Data::Inst().xform_i << "\n";
      break;
    }
    case '.':
    case '>':
    {
      Data::Inst().xform_i = (Data::Inst().xform_i + 1)%((int)(Data::Inst().GetXForms().size()));
      vcl_cout << "Xform: " << Data::Inst().xform_i << "\n";
      break;
    }
    case '0':
    {
       ScreenShot();
       break;
    }
  }
}

//*****************************************************************************

void ScreenShot()
{
  unsigned char *pixels = (unsigned char *)malloc(sizeof(unsigned char)*Data::Inst().window_width*Data::Inst().window_height*3);
  glReadPixels(0, 0, Data::Inst().window_width, Data::Inst().window_height, GL_RGB, GL_UNSIGNED_BYTE, pixels);

  vil_image_view<vil_rgb<vxl_byte> > img(Data::Inst().window_width, Data::Inst().window_height);

  
  for (int i = 0; i < Data::Inst().window_width; i++)
    for (int j = 0; j < Data::Inst().window_height; j++)  {
      int index = (j*Data::Inst().window_width + i)*3;
        img(i,Data::Inst().window_height-j-1) = vil_rgb<vxl_byte>(pixels[index],pixels[index+1],pixels[index+2]);
    }

  vcl_string image_name;
  vcl_cout << "Enter image name to save to: ";
  vcl_cin >> image_name;

  vil_save(img, image_name.c_str());
  free(pixels);

  vcl_cout << "Saved.\n";
}

//*****************************************************************************

