#include "dispkeypoint.h"

#include <vnl/vnl_math.h>
#include <GL/glut.h>


DispKeypoint::DispKeypoint(const DispKeypoint &k)
{
  ul = k.ul;
  ur = k.ur;
  ll = k.ll;
  lr = k.lr;
  id = k.id;
  origin = k.origin;
  xaxis = k.xaxis;
  yaxis = k.yaxis;
  zaxis = k.zaxis;
  scale = k.scale;
  feature_vector = k.feature_vector;
}

//*****************************************************************************

void DispKeypoint::DrawKeypoint(float r, float g, float b, float rb, float gb, float bb, float linewidth)
{
  glPushMatrix();
 /*   glBegin(GL_QUADS);
    glColor4f(rb, gb, bb, 0.3f);
    glVertex3d(ul[0], ul[1], ul[2]);
    glVertex3d(ur[0], ur[1], ur[2]);
    glVertex3d(lr[0], lr[1], lr[2]);
    glVertex3d(ll[0], ll[1], ll[2]); 
  glEnd();      */
 glDisable(GL_DEPTH_TEST);
     glLineWidth(linewidth);

  glBegin(GL_LINE_LOOP);

    glColor3f(r, g, b);
    glVertex3d(ul[0], ul[1], ul[2]);
    glVertex3d(ur[0], ur[1], ur[2]);
    glVertex3d(lr[0], lr[1], lr[2]);
    glVertex3d(ll[0], ll[1], ll[2]); 
  glEnd();      

  glLineWidth(linewidth/2.0f);  

  double dh = 4.0 * scale;

  // Scale the line segments drawn below so that the longest segment
  // is equal in length to the descriptor radius.
  double bin_scale = feature_vector.max_value()*1.15;

  if(bin_scale > 0)   
    bin_scale = 2.0*scale/bin_scale;
    
  // Draw the 4x4x8 histogram bin values organized as line segments
  // originating at the spatial bin centers.
  glBegin(GL_LINES);
  vnl_double_3 npt = origin + 4.0*scale*zaxis, dpt = origin + 4.0*scale*xaxis, bpt = origin + 4.0*scale*yaxis;
  //glVertex3d(origin[0], origin[1], origin[2]);  
  //glVertex3d(npt[0], npt[1], npt[2]);
  //glVertex3d(origin[0], origin[1], origin[2]);
  //glVertex3d(dpt[0], dpt[1], dpt[2]);
  //glVertex3d(origin[0], origin[1], origin[2]);
  //glVertex3d(bpt[0], bpt[1], bpt[2]);
   glEnd();

  glBegin(GL_LINES);
  glColor3f(r,g,b);

  for(int j=-1; j <= 2; ++j)
  {
    for(int i=-1; i <= 2; ++i)
    {
      // Compute the histogram center location.
      double x1 = dh*((i-0.5));
      double y1 = dh*((j-0.5));

      vnl_double_3 c1 = x1 * xaxis + y1 * yaxis + origin;

      const int n = 8;
      for(int bin=0; bin < n; ++bin)
      {
        // Orientation bins are indexed in clockwise order starting
        // with the x-axis in a standard coordinate system with the
        // origin in the lower left.  Compute the direction for this
        // bin.
        double angle = ((double)bin/n)*2.0*vnl_math::pi;
        vnl_double_3 ahat = vcl_cos(angle)*xaxis + vcl_sin(angle)*yaxis;

        // Scale the line segment by the feature vector component.
        double v = feature_vector[(j+1)*n*4 + (i+1)*n + bin];
        vnl_double_3 c2 = c1+v*bin_scale*ahat;

        glVertex3d(c1[0], c1[1], c1[2]);
        glVertex3d(c2[0], c2[1], c2[2]);
      }
    }
  }  
  glEnd();
  glEnable(GL_DEPTH_TEST);
  glPopMatrix();
}

//*****************************************************************************

void DispKeypoint::DrawFarKeypoint(float r, float g, float b)
{
  glPushMatrix();
  
 glDisable(GL_DEPTH_TEST);
     glLineWidth(3.0f);

  glBegin(GL_LINE_LOOP);
    glColor3f(r, g, b);
    glVertex3d(ul[0], ul[1], ul[2]);
    glVertex3d(ur[0], ur[1], ur[2]);
    glVertex3d(lr[0], lr[1], lr[2]);
    glVertex3d(ll[0], ll[1], ll[2]); 
  glEnd();      


  glLineWidth(2.0f);

  double dh = 4.0 * scale;

  // Scale the line segments drawn below so that the longest segment
  // is equal in length to the descriptor radius.
  double bin_scale = feature_vector.max_value()*1.15;

  if(bin_scale > 0)   
    bin_scale = 2.0*scale/bin_scale;
    
  glBegin(GL_LINES);
  for(int j=-1; j <= 2; ++j)
  {
    for(int i=-1; i <= 2; ++i)
    {
      // Compute the histogram center location.
      double x1 = dh*((i-0.5));
      double y1 = dh*((j-0.5));

      vnl_double_3 c1 = x1 * xaxis + y1 * yaxis + origin;

      const int n = 8;
      int largestindex = 0;
      double max = 0.0;
      for (int bin = 0; bin < n; ++bin)
      {
        double v = feature_vector[(j+1)*n*4 + (i+1)*n + bin];
        if (v > max)
        {
          max = v;
          largestindex = bin;
        }
      }
      
       // Orientation bins are indexed in clockwise order starting
      // with the x-axis in a standard coordinate system with the
      // origin in the lower left.  Compute the direction for this
      // bin.
      double angle = ((double)largestindex/n)*2.0*vnl_math::pi;
      vnl_double_3 ahat = vcl_cos(angle)*xaxis + vcl_sin(angle)*yaxis;

      // Scale the line segment by the feature vector component.
      double v = feature_vector[(j+1)*n*4 + (i+1)*n + largestindex];
      vnl_double_3 c2 = c1+v*bin_scale*ahat;

      glVertex3d(c1[0], c1[1], c1[2]);
      glVertex3d(c2[0], c2[1], c2[2]);
    }
  }

  glEnd();
  glEnable(GL_DEPTH_TEST);
  glPopMatrix();
}

//*****************************************************************************

void ReadKeypoints(vcl_vector<vcl_vector<DispKeypoint *> > &kpts, const char *filename)
{
  kpts.clear();
  FILE *file = fopen(filename, "rb");
  unsigned int numscales;
  fread(&numscales, sizeof(unsigned int), 1, file);

  kpts.resize(numscales);
  int count = 0;
  for (unsigned int i = 0; i < numscales; i++)
  {
    unsigned int size;
    fread(&size, sizeof(unsigned int), 1, file);
    kpts[i].resize(size);
    for (unsigned int j = 0; j < size; j++)
    {
      kpts[i][j] = new DispKeypoint;
      ReadKeypoint(file, kpts[i][j]);
      count++;
    }
  }

  fclose(file);
  vcl_cout << "Read " << count << " keypoints from " << filename << vcl_endl;
}

//*****************************************************************************

void ReadKeypoint(FILE *file, DispKeypoint *kp)
{
  fread(kp->origin.data_block(), sizeof(double)*3, 1, file);
  fread(kp->xaxis.data_block(), sizeof(double)*3, 1, file);
  fread(kp->zaxis.data_block(), sizeof(double)*3, 1, file);
  kp->yaxis = vnl_cross_3d(kp->xaxis, kp->zaxis);
  fread(&kp->scale, sizeof(double), 1, file);
  double strength;
  fread(&strength, sizeof(double), 1, file);
  fread(&kp->id, sizeof(int), 1, file); 
  fread(kp->feature_vector.data_block(), sizeof(double)*128, 1, file);

  fread(kp->ul.data_block(), sizeof(double)*3, 1, file);
  fread(kp->ur.data_block(), sizeof(double)*3, 1, file);
  fread(kp->ll.data_block(), sizeof(double)*3, 1, file);
  fread(kp->lr.data_block(), sizeof(double)*3, 1, file);
}

//*****************************************************************************
