//Eric Smith
//smithe4@gmail.com

#pragma warning( disable : 4996 )

#include <vnl/vnl_double_3.h>
#include <vnl/vnl_float_3.h>

#include <vcl_iostream.h>
#include <GL/glut.h>

#include "Display.h"
#include "Input.h"
#include "mesh.h"
#include "data.h"

#include <vcl_string.h>
#include <vcl_sstream.h>
#include <vcl_fstream.h>

#include <vul/vul_arg.h>
#include <vul/vul_file.h>

#include <vcl_algorithm.h>

using namespace std;

void InitializeGL();
void LoadXforms(const char *filename, vcl_vector<xform> &xforms);
bool ReadMatches(vcl_vector<dispkptmatch> &matches, const vcl_vector< vcl_vector<DispKeypoint *> > &fixedkpts,
                 const vcl_vector< vcl_vector<DispKeypoint *> > &movingkpts, const char *filename);

//*****************************************************************************

int main(int argc, char* argv[])
{
  // Initialize OpenGL
  glutInit(&argc, argv);
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH);
  
  //Windowed Mode Stuff
  glutInitWindowSize(Data::Inst().window_width, Data::Inst().window_height);
  glutInitWindowPosition(0, 0);
  glutCreateWindow("XForm Viewer");

  vul_arg<vcl_string> path1(0, "Full path to the first scan mesh(*.vtp file)");
  vul_arg<vcl_string> path2(0, "Full path to the second scan mesh(*.vtp file)");
  vul_arg_parse(argc, argv);

  vcl_string dir1 = vul_file::dirname(path1());
  vcl_string dir2 = vul_file::dirname(path2());
  vcl_string name1 = vul_file::strip_extension(vul_file::strip_directory(path1()));

  Data::Inst().fixedmesh.SetColor(1.0f, 0.0f, 0.0f, 0.8f);
  Data::Inst().movingmesh.SetColor(0.0f, 0.0f, 1.0f, 0.8f);
  Data::Inst().fixedmesh.LoadMesh(path1());
  Data::Inst().movingmesh.LoadMesh(path2());
  Data::Inst().fixedmesh.LookAt();

  vcl_vector<vcl_vector<DispKeypoint *> > fixedkpts, movingkpts;
  ReadKeypoints(fixedkpts, (dir1 + "/pskpts.dat").c_str());
  ReadKeypoints(movingkpts, (dir2 + "/pskpts.dat").c_str());
  ReadMatches(Data::Inst().matches, fixedkpts, movingkpts, (dir2 + "/matches_to_" + name1 + ".dat").c_str());
  LoadXforms((dir2 + "/xforms_to_" + name1 + ".txt").c_str(), Data::Inst().xforms);
  InitializeGL();

  // Set up all of the callback functions.
  glutReshapeFunc(ReshapeFunction);
  glutKeyboardFunc(KeyboardFunction);
  glutDisplayFunc(DisplayFunction);
  glutIdleFunc(IdleFunction);
  glutKeyboardUpFunc(KeyboardUpFunction);
  glutMouseFunc(MouseFunction);
  glutMotionFunc(MotionFunction);
  glutTimerFunc(50, TimerFunction, 0);

  glutMainLoop();

  return 0;
}

//*****************************************************************************

void InitializeGL()
{  
  //Depth testing and alpha testing enabled
  glEnable(GL_DEPTH_TEST);
  glEnable(GL_ALPHA_TEST);
  glEnable(GL_LINE_SMOOTH);
  glAlphaFunc(GL_GREATER, 0);
  glEnable(GL_BLEND);
  glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  glLineWidth(3.0f);
  glClearColor(0.5f, 0.5f, 0.9f, 1.0f);
  //glClearColor(0.2f, 0.2f, 0.2f, 1.0f);
}

//*****************************************************************************

void LoadXforms(const char *filename, vcl_vector<xform> &xforms)
{
  vcl_ifstream infile(filename);
    
  vcl_string x;

  std::getline(infile, x);
  std::getline(infile, x);
  std::getline(infile, x);
  
  while (infile.good())
  {
    xform xf;
    std::getline(infile, x);
    if (x.empty())
      break;

    vcl_istringstream ss(x);

    vnl_double_3 trans;
    ss >> xf.t;
    
    //Get rotation
    vnl_double_3 r1, r2, r3;
    ss >> r1 >> r2 >> r3;

    xf.R.set_row(0, r1);
    xf.R.set_row(1, r2);
    xf.R.set_row(2, r3);  

    xf.mxform[0] = (float)r1[0];    xf.mxform[4] = (float)r1[1];    xf.mxform[8] = (float)r1[2];
    xf.mxform[1] = (float)r2[0];    xf.mxform[5] = (float)r2[1];    xf.mxform[9] = (float)r2[2];
    xf.mxform[2] = (float)r3[0];    xf.mxform[6] = (float)r3[1];    xf.mxform[10] = (float)r3[2];
    xf.mxform[12] = (float)xf.t[0];    
    xf.mxform[13] = (float)xf.t[1];    
    xf.mxform[14] = (float)xf.t[2];
    xf.mxform[3] = xf.mxform[7] = xf.mxform[11] = 0.0f;
    xf.mxform[15] = 1.0f;

    vnl_double_3 i_t = - xf.R.transpose() * xf.t;
    vnl_double_3x3 i_R = xf.R.transpose();

    xf.imxform[0] = (float)i_R(0,0);    xf.imxform[4] = (float)i_R(0,1);    xf.imxform[8] = (float)i_R(2,0);
    xf.imxform[1] = (float)i_R(1,0);    xf.imxform[5] = (float)i_R(1,1);    xf.imxform[9] = (float)i_R(2,1);
    xf.imxform[2] = (float)i_R(2,0);    xf.imxform[6] = (float)i_R(2,1);    xf.imxform[10] = (float)i_R(2,2);
    xf.imxform[12] = (float)i_t[0];    
    xf.imxform[13] = (float)i_t[1];    
    xf.imxform[14] = (float)i_t[2];
    xf.imxform[3] = xf.imxform[7] = xf.imxform[11] = 0.0f;
    xf.imxform[15] = 1.0f;

    xforms.push_back(xf);
  }

  vcl_cout << "Read " << xforms.size() << " xforms.\n";

  infile.close();
}

//*****************************************************************************

bool ReadMatches(vcl_vector<dispkptmatch> &matches, const vcl_vector< vcl_vector<DispKeypoint *> > &fixedkpts,
                 const vcl_vector< vcl_vector<DispKeypoint *> > &movingkpts, const char *filename)
{
  FILE *file = fopen(filename, "rb");
  if (!file)
    return false;

  unsigned int size;
  fread(&size, sizeof(unsigned int), 1, file);
  matches.resize(size);
  for (unsigned int i = 0; i < size; i++)
  {
    int sindex;
    fread(&sindex, sizeof(int), 1, file);
    int fid, mid, bid;
    fread(&fid, sizeof(int), 1, file);
    fread(&mid, sizeof(int), 1, file);
    fread(&bid, sizeof(int), 1, file);
    matches[i].f = fixedkpts[sindex][fid];
    matches[i].m = movingkpts[sindex][mid];
    matches[i].b = fixedkpts[sindex][bid];

    fread(&matches[i].ratio, sizeof(double), 1, file);
    fread(&matches[i].range_error, sizeof(double), 1, file);
    fread(&matches[i].index, sizeof(int), 1, file);
    fread(&matches[i].sortby, sizeof(double), 1, file);
  }

  fclose(file);
  vcl_cout << "Read " << size << " matches from " << filename << vcl_endl;
  return true;
}

//*****************************************************************************
