#include "mesh.h"

#include <vcl_iostream.h>
#include "vtkPolyData.h"
#include "vtkPointData.h"
#include "vtkXMLPolyDataReader.h"
#include "vtkFloatArray.h"
#include "vtkCellArray.h"
#include "vtkSmartPointer.h"
#include "data.h"

#include <GL/glut.h>

//*****************************************************************************

Mesh::Mesh() :  displaylist(-1), r_(1.0f), g_(1.0f), b_(1.0f), a_(1.0f), usesolidcolor(false) 
{

}


//*****************************************************************************

void Mesh::Draw()
{
  glCallList(displaylist);
}

//*****************************************************************************

void Mesh::SetColor(float r, float g, float b, float a)
{
  r_ = r;
  g_ = g;
  b_ = b;
  a_ = a;
}

//*****************************************************************************

void Mesh::LoadMesh(const vcl_string &filename)
{
  if (displaylist != -1)
    glDeleteLists(displaylist, 1);

  vcl_cout << "Loading " << filename << "\n";

  vtkSmartPointer<vtkXMLPolyDataReader> polyreader = vtkSmartPointer<vtkXMLPolyDataReader>::New();
  polyreader->SetFileName(filename.c_str());
  polyreader->Update();
  vtkPolyData *polydata = polyreader->GetOutput();
 
  triangles.clear();
  points.clear();
  colors.clear();

  center = vnl_double_3(0.0, 0.0, 0.0);
  vtkPoints *pts = (vtkPoints *)polydata->GetPoints();
  for (int i = 0; i < polydata->GetNumberOfPoints(); i++)
  {
    vnl_double_3 pt;
    pt.set(polydata->GetPoint(i));
    center += pt;
    points.push_back(pt);
  }

  center /= (double)points.size();

  vtkUnsignedCharArray *color = (vtkUnsignedCharArray *)polydata->GetPointData()->GetVectors("Colors");
  for (vtkIdType i = 0; i < color->GetNumberOfTuples(); i++)  {
    unsigned char tuple[3];
    color->GetTupleValue(i, tuple);
    vcl_vector<unsigned char> tupvec(3);
    tupvec[0] = tuple[0]; tupvec[1] = tuple[1]; tupvec[2] = tuple[2];
    colors.push_back(tupvec);  
  }

  for (int i = 0; i < polydata->GetNumberOfCells(); i++)
  {
    vnl_int_3 triangle;
    triangle[0] = polydata->GetCell(i)->GetPointId(0);
    triangle[1] = polydata->GetCell(i)->GetPointId(1);
    triangle[2] = polydata->GetCell(i)->GetPointId(2);
    triangles.push_back(triangle);
  }

  displaylist = glGenLists(1);
  glNewList(displaylist, GL_COMPILE);
    glBegin(GL_TRIANGLES);
      for (unsigned int i = 0; i < triangles.size(); i++)
      {
        vnl_int_3 &t = triangles[i];
        glColor3ub(colors[t[0]][0],  colors[t[0]][1], colors[t[0]][2]);
        glVertex3d(points[t[0]][0], points[t[0]][1], points[t[0]][2]);
        glColor3ub(colors[t[1]][0],  colors[t[1]][1], colors[t[1]][2]);
        glVertex3d(points[t[1]][0], points[t[1]][1], points[t[1]][2]);
        glColor3ub(colors[t[2]][0],  colors[t[2]][1], colors[t[2]][2]);
        glVertex3d(points[t[2]][0], points[t[2]][1], points[t[2]][2]);      
      }
    glEnd();
  glEndList();
}

//*****************************************************************************

void Mesh::FlipColor()
{
  usesolidcolor = !usesolidcolor;

  if (displaylist != -1)
    glDeleteLists(displaylist, 1);

  displaylist = glGenLists(1);
  glNewList(displaylist, GL_COMPILE);
    glBegin(GL_TRIANGLES);
      for (unsigned int i = 0; i < triangles.size(); i++)
      {
        vnl_int_3 &t = triangles[i];
        glColor4f(r_, g_, b_, a_);
        if (!usesolidcolor) glColor3ub(colors[t[0]][0],  colors[t[0]][1], colors[t[0]][2]);
        glVertex3d(points[t[0]][0], points[t[0]][1], points[t[0]][2]);
        if (!usesolidcolor) glColor3ub(colors[t[1]][0],  colors[t[1]][1], colors[t[1]][2]);
        glVertex3d(points[t[1]][0], points[t[1]][1], points[t[1]][2]);
        if (!usesolidcolor) glColor3ub(colors[t[2]][0],  colors[t[2]][1], colors[t[2]][2]);
        glVertex3d(points[t[2]][0], points[t[2]][1], points[t[2]][2]);
      }
    glEnd();
  glEndList();
}

//*****************************************************************************

void Mesh::LookAt()
{
  vnl_double_3 pz = -center;
  pz[2] = 0.0f;
  pz.normalize();
  Data::Inst().eyex = 0.0; 
  Data::Inst().eyey = 0.0; 
  Data::Inst().eyez = 0.0;
  Data::Inst().eyeangle = atan2(-pz[1], -pz[0]);
}

//*****************************************************************************
